﻿using Projekt.Framework;
using Projekt.Interfaces;
using System.Data;

namespace Projekt.Data.Access
{
    

    public class Mitglied : IAccess<Entities.Mitglied, Parameters.Mitglied>
    {
        private bool isRead;
        private List<SQLParameter>? Parameters { get; set; }

        public string GetParameters(Parameters.Mitglied? parameters)
        {
            string Params = string.Empty;
            Parameters = new List<SQLParameter>(); 

            if (parameters != null)
            {
                if (parameters.Id != null && parameters.Id > 0)
                {
                    Params.AddLine(" AND Id = @Id ");
                    Parameters.Add(new SQLParameter("@Id", parameters.Id));
                }

                if (parameters.Vorname != null && !string.IsNullOrWhiteSpace(parameters.Vorname))
                {
                    Params.AddLine(" AND Vorname = @Vorname ");
                    Parameters.Add(new SQLParameter("@Vorname", parameters.Vorname));
                }

                if (parameters.Nachname != null && !string.IsNullOrWhiteSpace(parameters.Nachname))
                {
                    Params.AddLine(" AND Nachname = @Nachname ");
                    Parameters.Add(new SQLParameter("@Nachname", parameters.Nachname));
                }

                if (parameters.Land != null && !string.IsNullOrWhiteSpace(parameters.Land))
                {
                    Params.AddLine(" AND Land = @Land ");
                    Parameters.Add(new SQLParameter("@Land", parameters.Land));
                }

                if (parameters.Straße != null && !string.IsNullOrWhiteSpace(parameters.Straße))
                {
                    Params.AddLine(" AND Straße = @Straße ");
                    Parameters.Add(new SQLParameter("@Straße", parameters.Straße));
                }

                if (parameters.Hausnummer != null && !string.IsNullOrWhiteSpace(parameters.Hausnummer))
                {
                    Params.AddLine(" AND Hausnummer = @Hausnummer ");
                    Parameters.Add(new SQLParameter("@Hausnummer", parameters.Hausnummer));
                }

                if (parameters.Emailadresse != null && !string.IsNullOrWhiteSpace(parameters.Emailadresse))
                {
                    Params.AddLine(" AND Emailadresse = @Emailadresse ");
                    Parameters.Add(new SQLParameter("@Emailadresse", parameters.Emailadresse));
                }

                if (parameters.Telefon != null && !string.IsNullOrWhiteSpace(parameters.Telefon))
                {
                    Params.AddLine(" AND Telefon = @Telefon ");
                    Parameters.Add(new SQLParameter("@Telefon", parameters.Telefon));
                }
            }

            if (string.IsNullOrWhiteSpace(Params) && !isRead)
                return "AND 1 = 2 ";
            else
                return Params;

        }

        public List<Entities.Mitglied> Get(Parameters.Mitglied? parameters)
        {
            DataTable? ResultSet = new DataTable();
            List<Entities.Mitglied> ret = new List<Entities.Mitglied>();

            isRead = true;
            string Query = string.Empty;
            if (Parameters == null || Parameters.Count > 0)
            {
                Parameters = new List<SQLParameter>();
            }
            
            Query.AddLine("SELECT * ");
            Query.AddLine("FROM MitgliedmitInfos ");
            Query.AddLine("WHERE ");
            Query.AddLine("1 = 1 ");
            Query.AddLine(GetParameters(parameters));
            
            if (Common.Context.SQL != null)
            {
                ResultSet = Common.Context.SQL.GetResultSet(Query, Parameters, false);
            }
            else
            {
                throw new ArgumentException("Datenbankzugriff nicht wurde initialisiert.");
            }
            

            if (ResultSet != null && ResultSet.Rows.Count > 0)
                ret = (from DataRow x in ResultSet.Rows
                       select new Entities.Mitglied()
                       {
                           Id = Convert.IsDBNull(x["Id"]) ? 0 : Convert.ToInt32(x["Id"]),
                           Vorname = Convert.IsDBNull(x["Vorname"]) ? string.Empty : Convert.ToString(x["Vorname"]),
                           Nachname = Convert.IsDBNull(x["Nachname"]) ? string.Empty : Convert.ToString(x["Nachname"]),
                           Land = Convert.IsDBNull(x["Land"]) ? string.Empty : Convert.ToString(x["Land"]),
                           Straße = Convert.IsDBNull(x["Straße"]) ? string.Empty : Convert.ToString(x["Straße"]),
                           Hausnummer = Convert.IsDBNull(x["Hausnummer"]) ? string.Empty : Convert.ToString(x["Hausnummer"]),
                           Emailadresse = Convert.IsDBNull(x["Emailadresse"]) ? string.Empty : Convert.ToString(x["Emailadresse"]),
                           Telefon = Convert.IsDBNull(x["Telefon"]) ? string.Empty : Convert.ToString(x["Telefon"])
                       }).ToList<Entities.Mitglied>();

            return ret;
        }

        public bool Save(Entities.Mitglied entity)
        {
            Parameters = new List<SQLParameter>
            {
                new SQLParameter("@Id", entity.Id),
                new SQLParameter("@Vorname", entity.Vorname),
                new SQLParameter("@Nachname", entity.Nachname),
                new SQLParameter("@Land", entity.Land),
                new SQLParameter("@Straße", entity.Straße),
                new SQLParameter("@Hausnummer", entity.Hausnummer),
                new SQLParameter("@Emailadresse", entity.Emailadresse),
                new SQLParameter("@Telefon", entity.Telefon)
            };

            string Statement = string.Empty;
            Statement.AddLine("IF NOT EXISTS ");
            Statement.AddLine("(SELECT * FROM MitgliedmitInfos ");
            Statement.AddLine("WHERE 1 = 1 ");
            Statement.AddLine("AND Id = @Id");
            Statement.AddLine(") ");
            Statement.AddLine("INSERT INTO MitgliedmitInfos ");
            Statement.AddLine("( ");
            Statement.AddLine("Vorname, ");
            Statement.AddLine("Nachname, ");
            Statement.AddLine("Land, ");
            Statement.AddLine("Straße, ");
            Statement.AddLine("Hausnummer, ");
            Statement.AddLine("Emailadresse, ");
            Statement.AddLine("Telefon");
            Statement.AddLine(") ");
            Statement.AddLine("VALUES ");
            Statement.AddLine("( ");
            Statement.AddLine("@Vorname, ");
            Statement.AddLine("@Nachname, ");
            Statement.AddLine("@Land, ");
            Statement.AddLine("@Straße, ");
            Statement.AddLine("@Hausnummer, ");
            Statement.AddLine("@Emailadresse, ");
            Statement.AddLine("@Telefon");
            Statement.AddLine(") ");
            Statement.AddLine("ELSE ");
            Statement.AddLine("UPDATE MitgliedmitInfos ");
            Statement.AddLine("SET ");
            Statement.AddLine("Vorname = @Vorname, ");
            Statement.AddLine("Nachname = @Nachname, ");
            Statement.AddLine("Land = @Land, ");
            Statement.AddLine("Straße = @Straße, ");
            Statement.AddLine("Hausnummer = @Hausnummer, ");
            Statement.AddLine("Emailadresse = @Emailadresse, ");
            Statement.AddLine("Telefon = @Telefon");
            Statement.AddLine("WHERE 1 = 1 ");
            Statement.AddLine("AND Id = @Id");
            
            bool Successful = false;
            if (Common.Context.SQL != null)
            {
                Successful = Common.Context.SQL.Execute(Statement, Parameters, false);
            }
            else
            {
                throw new ArgumentException("Datenbankzugriff nicht wurde initialisiert.");
            }
           
            if(Successful && Common.Context.SQL.Identity != null)
            {
                entity.Id = Common.Context.SQL.Identity.Value;
            }

            return Successful;
        }

        bool IAccess<Entities.Mitglied, Parameters.Mitglied>.Delete(Parameters.Mitglied? parameters)
        {
            isRead = false;           
            string Statement = string.Empty;
            Statement.AddLine("DELETE ");
            Statement.AddLine("FROM MitgliedmitInfos ");
            Statement.AddLine("WHERE ");
            Statement.AddLine("1 = 1 ");
            Statement.AddLine(GetParameters(parameters));
            if (Common.Context.SQL != null)
            {
                return Common.Context.SQL.Execute(Statement, Parameters, false);
            }
            else
            {
                throw new ArgumentException("Datenbankzugriff nicht wurde initialisiert.");
            }
        }

        public bool Delete(Entities.Mitglied entity)
        {

            isRead = false;

            Parameters = new List<SQLParameter>
            {
                { new SQLParameter("@Id", entity.Id) }
            };

            string Statement = string.Empty;
            Statement.AddLine("DELETE ");
            Statement.AddLine("FROM MitgliedmitInfos ");
            Statement.AddLine("WHERE ");
            Statement.AddLine("1 = 1 ");
            Statement.AddLine("AND Id = @Id");
            if (Common.Context.SQL != null)
            {
                return Common.Context.SQL.Execute(Statement, Parameters, false);
            }
            else
            {
                throw new ArgumentException("Datenbankzugriff nicht wurde initialisiert.");
            }
        }

        public static Mitglied Create()
        {
            return new Mitglied();
        }

    }
}
