﻿using System.Data;
using System.Data.SqlClient;

namespace Projekt.Framework
{
    public  class DatabaseAccess
    {
        public int Timeout { get; set; }
        public Nullable<int> Identity { get; set; }

        public DataTable? GetResultSet(string query, List<SQLParameter>? parameters, bool silent, Configuration? setting = null)
        {
            try
            {
                DataTable result = new DataTable();
                SqlCredential? cred = null;

                string constring = string.Empty;
                if (setting == null)
                {
                    constring = Common.Context.Default?.GetConnectionString() ?? throw new ArgumentException("Keine Verbindungsdaten für SQL-Server hinterlegt.");
                    cred = Common.Context.Default.credentials;
                }
                else
                {
                    constring = setting.GetConnectionString();
                    cred = setting.credentials;
                }

                using (SqlConnection connection = new SqlConnection(constring, cred))
                {
                    if (!(query.ToUpper().Contains("CREATE ") || query.ToUpper().Contains("ENABLE ") || query.ToUpper().Contains("DISABLE ")))
                    {
                        string newquery = "SET ARITHABORT ON";
                        newquery.AddLine("SET DATEFORMAT dmy");
                        newquery.AddLine(query);
                        query = newquery;
                    }
                    SqlCommand command = connection.CreateCommand();
                    command.CommandType = CommandType.Text;
                    command.CommandTimeout = Timeout;
                    Timeout = 120;
                    command.CommandText = query;

                    if (parameters != null && parameters.Count > 0)
                    {
                        foreach (SQLParameter parameter in parameters)
                        {
                            if (parameter.Value != null)
                            {
                                command.Parameters.Add(parameter.GetSqlParameter());
                            }
                            else
                            {
                                command.Parameters.AddWithValue(parameter.Name, DBNull.Value);
                            }
                        }
                    }

                    if (connection.State != ConnectionState.Open)
                    {
                        connection.Open();
                    }

                    using (SqlDataAdapter reader = new SqlDataAdapter(command))
                    {
                        result.BeginLoadData();
                        reader.Fill(result);
                        result.EndLoadData();
                    }
                }
                return result;
            }
            catch
            {
                //Logging
                return null;
            }
        }

        public object? GetResult(string query, dynamic defaultvalue, List<SQLParameter>? parameters, bool silent, Configuration? setting = null)
        {
            try
            {
                object result;
                SqlCredential? cred = null;
                string constring = string.Empty;
                if (setting == null)
                {
                    constring = Common.Context.Default?.GetConnectionString() ?? throw new ArgumentException("Keine Verbindungsdaten für SQL-Server hinterlegt.");
                    cred = Common.Context.Default.credentials;
                }
                else
                {
                    constring = setting.GetConnectionString();
                    cred = setting.credentials;
                }

                using (SqlConnection connection = new SqlConnection(constring, cred))
                {
                    if (!(query.ToUpper().Contains("CREATE ") || query.ToUpper().Contains("ENABLE ") || query.ToUpper().Contains("DISABLE ")))
                    {
                        string newquery = "SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED";
                        newquery.AddLine("SET DATEFORMAT dmy");
                        newquery.AddLine(query);
                        query = newquery;
                    }
                    SqlCommand command = connection.CreateCommand();
                    command.CommandType = CommandType.Text;
                    command.CommandTimeout = Timeout;
                    Timeout = 120;
                    command.CommandText = query;

                    if (parameters != null && parameters.Count > 0)
                    {
                        foreach (SQLParameter parameter in parameters)
                        {
                            if (parameter.Value != null)
                            {
                                command.Parameters.Add(parameter.GetSqlParameter());
                            }
                            else
                            {
                                command.Parameters.AddWithValue(parameter.Name, DBNull.Value);
                            }
                        }
                    }

                    if (connection.State != ConnectionState.Open)
                    {
                        connection.Open();
                    }

                    SqlDataReader reader = command.ExecuteReader(CommandBehavior.SingleRow);


                    if (reader == null || Convert.IsDBNull(reader) || !reader.HasRows)
                    {
                        result = defaultvalue;
                    }
                    else
                    {
                        reader.Read();
                        if (reader.GetValue(0) == null || Convert.IsDBNull(reader.GetValue(0)))
                        {
                            result = defaultvalue;
                        }
                        else
                        {
                            result = reader.GetValue(0);
                        }
                    }

                    connection.Close();
                    connection.Dispose();
                }
                return result;
            }
            catch
            {
                //Logging
                return null;
            }
        }

        public bool Execute(string statement, List<SQLParameter>? parameters, bool silent, Configuration? setting = null)
        {
            try
            {
                SqlCredential? cred = null;
                string constring = string.Empty;
                if (setting == null)
                {
                    constring = Common.Context.Default?.GetConnectionString() ?? throw new ArgumentException("Keine Verbindungsdaten für SQL-Server hinterlegt.");
                    cred = Common.Context.Default.credentials;
                }
                else
                {
                    constring = setting.GetConnectionString();
                    cred = setting.credentials;
                }

                using (SqlConnection connection = new SqlConnection(constring, cred))
                {
                    if (!(statement.ToUpper().Contains("CREATE ") || statement.ToUpper().Contains("ENABLE ") || statement.ToUpper().Contains("DISABLE ")))
                    {
                        string newquery = "SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED ";
                        newquery.AddLine("SET DATEFORMAT dmy ");
                        newquery.AddLine(statement);
                        statement = newquery;
                    }
                    SqlCommand command = connection.CreateCommand();
                    command.CommandType = CommandType.Text;
                    command.CommandTimeout = Timeout;
                    Timeout = 120;
                    command.CommandText = statement;

                    if (parameters != null && parameters.Count > 0)
                    {
                        foreach (SQLParameter parameter in parameters)
                        {
                            if (parameter.Value != null)
                            {
                                command.Parameters.Add(parameter.GetSqlParameter());
                            }
                            else
                            {
                                command.Parameters.AddWithValue(parameter.Name, DBNull.Value);
                            }
                        }
                    }

                    if (connection.State != ConnectionState.Open)
                    {
                        connection.Open();
                    }
                    int affected = command.ExecuteNonQuery();
                    command.CommandText = "SELECT @@Identity";
                    object ExecuteScalarValue = command.ExecuteScalar();

                    if (!Convert.IsDBNull(ExecuteScalarValue) && ExecuteScalarValue != null)
                    {
                        Identity = Convert.ToInt32(ExecuteScalarValue);
                    }
                    else
                    {
                        Identity = null;
                    }
                }
                return true;
            }
            catch
            {
                //Logging
                return false;
            }           
        }
    }
}

