﻿using System.Data.SqlClient;

namespace Projekt.Framework
{
    public class SQLParameter
    {
        public string Name { get; set; }
        public object? Value { get; set; }

        public SQLParameter(string name, object? value)
        {
            Name = name; 
            Value = value;
        }

        public SqlParameter GetSqlParameter()
        {
            SqlParameter Param = new SqlParameter();
            if (Name.StartsWith("@")) { Param.ParameterName = Name; }
            else { Param.ParameterName = "@" + Name; }

            if (Value == null)
            {
                Param.SqlDbType = System.Data.SqlDbType.Variant;
                Param.Value = DBNull.Value;
                return Param;
            }
            else
            {
                switch (Value.GetType().Name.ToLower())
                {
                    case "string":
                        Param.SqlDbType = System.Data.SqlDbType.VarChar;
                        Param.Value = Value.ToString();
                        break;
                    case "char":
                        Param.SqlDbType = System.Data.SqlDbType.VarChar;
                        Param.Value = Value.ToString();
                        break;
                    case "int":
                        Param.SqlDbType = System.Data.SqlDbType.Int;
                        Param.Value = Convert.ToInt32(Value);
                        break;
                    case "int32":
                        Param.SqlDbType = System.Data.SqlDbType.Int;
                        Param.Value = Convert.ToInt32(Value);
                        break;
                    case "long":
                        Param.SqlDbType = System.Data.SqlDbType.BigInt;
                        Param.Value = Convert.ToInt64(Value);
                        break;
                    case "int64":
                        Param.SqlDbType = System.Data.SqlDbType.BigInt;
                        Param.Value = Convert.ToInt64(Value);
                        break;
                    case "short":
                        Param.SqlDbType = System.Data.SqlDbType.SmallInt;
                        Param.Value = Convert.ToInt16(Value);
                        break;
                    case "int16":
                        Param.SqlDbType = System.Data.SqlDbType.SmallInt;
                        Param.Value = Convert.ToInt16(Value);
                        break;
                    case "bool":
                        Param.SqlDbType = System.Data.SqlDbType.Bit;
                        Param.Value = Convert.ToBoolean(Value);
                        break;
                    case "datetime":
                        Param.SqlDbType = System.Data.SqlDbType.DateTime;
                        Param.Value = Convert.ToDateTime(Value);
                        break;
                    case "double":
                        Param.SqlDbType = System.Data.SqlDbType.Decimal;
                        Param.Value = Convert.ToDouble(Value);
                        break;
                    case "single":
                        Param.SqlDbType = System.Data.SqlDbType.Decimal;
                        Param.Value = Convert.ToDouble(Value);
                        break;
                    case "decimal":
                        Param.SqlDbType = System.Data.SqlDbType.Decimal;
                        Param.Value = Convert.ToDouble(Value);
                        break;
                    default:
                        if (Value.GetType().Name.ToLower().StartsWith("nullable"))
                        {
                            switch (Value.GetType().UnderlyingSystemType.Name.ToLower())
                            {
                                case "string":
                                    Param.SqlDbType = System.Data.SqlDbType.VarChar;
                                    Param.Value = Value.ToString();
                                    break;
                                case "char":
                                    Param.SqlDbType = System.Data.SqlDbType.VarChar;
                                    Param.Value = Value.ToString();
                                    break;
                                case "int":
                                    Param.SqlDbType = System.Data.SqlDbType.Int;
                                    Param.Value = Convert.ToInt32(Value);
                                    break;
                                case "int32":
                                    Param.SqlDbType = System.Data.SqlDbType.Int;
                                    Param.Value = Convert.ToInt32(Value);
                                    break;
                                case "long":
                                    Param.SqlDbType = System.Data.SqlDbType.BigInt;
                                    Param.Value = Convert.ToInt64(Value);
                                    break;
                                case "int64":
                                    Param.SqlDbType = System.Data.SqlDbType.BigInt;
                                    Param.Value = Convert.ToInt64(Value);
                                    break;
                                case "short":
                                    Param.SqlDbType = System.Data.SqlDbType.SmallInt;
                                    Param.Value = Convert.ToInt16(Value);
                                    break;
                                case "int16":
                                    Param.SqlDbType = System.Data.SqlDbType.SmallInt;
                                    Param.Value = Convert.ToInt16(Value);
                                    break;
                                case "bool":
                                    Param.SqlDbType = System.Data.SqlDbType.Bit;
                                    Param.Value = Convert.ToBoolean(Value);
                                    break;
                                case "datetime":
                                    Param.SqlDbType = System.Data.SqlDbType.DateTime;
                                    Param.Value = Convert.ToDateTime(Value);
                                    break;
                                case "double":
                                    Param.SqlDbType = System.Data.SqlDbType.Decimal;
                                    Param.Value = Convert.ToDouble(Value);
                                    break;
                                case "single":
                                    Param.SqlDbType = System.Data.SqlDbType.Decimal;
                                    Param.Value = Convert.ToDouble(Value);
                                    break;
                                case "decimal":
                                    Param.SqlDbType = System.Data.SqlDbType.Decimal;
                                    Param.Value = Convert.ToDouble(Value);
                                    break;
                                default:
                                    Param.SqlDbType = System.Data.SqlDbType.VarChar;
                                    Param.Value = Value.ToString();
                                    break;
                            }
                        }
                        else
                        {
                            Param.SqlDbType = System.Data.SqlDbType.VarChar;
                            Param.Value = Value.ToString();
                            break;
                        }
                        break;
                }
            }
            return Param;
        }
    }
}
